@@ -34,14 +34,18 @@ class union_map:
             self.ctx = keywords["ctx"]
             self.ptr = keywords["ptr"]
             return
-        if len(args) == 1 and args[0].__class__ is map:
-            self.ctx = Context.getDefaultInstance()
-            self.ptr = isl.isl_union_map_from_map(isl.isl_map_copy(args[0].ptr))
-            return
         if len(args) == 1 and type(args[0]) == str:
             self.ctx = Context.getDefaultInstance()
             self.ptr = isl.isl_union_map_read_from_str(self.ctx, args[0])
             return
+        if len(args) == 1 and args[0].__class__ is basic_map:
+            self.ctx = Context.getDefaultInstance()
+            self.ptr = isl.isl_union_map_from_basic_map(isl.isl_basic_map_copy(args[0].ptr))
+            return
+        if len(args) == 1 and args[0].__class__ is map:
+            self.ctx = Context.getDefaultInstance()
+            self.ptr = isl.isl_union_map_from_map(isl.isl_map_copy(args[0].ptr))
+            return
         raise Error
     def __del__(self):
         if hasattr(self, 'ptr'):
@@ -53,6 +57,31 @@ class union_map:
         return res
     def __repr__(self):
         return 'isl.union_map("%s")' % str(self)
+    def is_strict_subset(self, arg1):
+        try:
+            if not arg1.__class__ is union_map:
+                arg1 = union_map(arg1)
+        except:
+            raise
+        res = isl.isl_union_map_is_strict_subset(self.ptr, arg1.ptr)
+        return res
+    def foreach_map(self, arg1):
+        exc_info = [None]
+        fn = CFUNCTYPE(c_int, c_void_p, c_void_p)
+        def cb_func(cb_arg0, cb_arg1):
+            cb_arg0 = map(ctx=self.ctx, ptr=cb_arg0)
+            try:
+                arg1(cb_arg0)
+            except:
+                import sys
+                exc_info[0] = sys.exc_info()
+                return -1
+            return 0
+        cb = fn(cb_func)
+        res = isl.isl_union_map_foreach_map(self.ptr, cb, None)
+        if exc_info[0] != None:
+            raise exc_info[0][0], exc_info[0][1], exc_info[0][2]
+        return res
     def affine_hull(self):
         res = isl.isl_union_map_affine_hull(isl.isl_union_map_copy(self.ptr))
         return union_map(ctx=self.ctx, ptr=res)
@@ -148,6 +177,22 @@ class union_map:
             raise
         res = isl.isl_union_map_intersect_range(isl.isl_union_map_copy(self.ptr), isl.isl_union_set_copy(arg1.ptr))
         return union_map(ctx=self.ctx, ptr=res)
+    def subtract_domain(self, arg1):
+        try:
+            if not arg1.__class__ is union_set:
+                arg1 = union_set(arg1)
+        except:
+            raise
+        res = isl.isl_union_map_subtract_domain(isl.isl_union_map_copy(self.ptr), isl.isl_union_set_copy(arg1.ptr))
+        return union_map(ctx=self.ctx, ptr=res)
+    def subtract_range(self, arg1):
+        try:
+            if not arg1.__class__ is union_set:
+                arg1 = union_set(arg1)
+        except:
+            raise
+        res = isl.isl_union_map_subtract_range(isl.isl_union_map_copy(self.ptr), isl.isl_union_set_copy(arg1.ptr))
+        return union_map(ctx=self.ctx, ptr=res)
     def apply_domain(self, arg1):
         try:
             if not arg1.__class__ is union_map:
@@ -168,7 +213,7 @@ class union_map:
         res = isl.isl_union_map_reverse(isl.isl_union_map_copy(self.ptr))
         return union_map(ctx=self.ctx, ptr=res)
     def detect_equalities(self):
-        res = isl.isl_union_map_detect_equalities(self.ptr)
+        res = isl.isl_union_map_detect_equalities(isl.isl_union_map_copy(self.ptr))
         return union_map(ctx=self.ctx, ptr=res)
     def deltas(self):
         res = isl.isl_union_map_deltas(isl.isl_union_map_copy(self.ptr))
@@ -201,36 +246,13 @@ class union_map:
             raise
         res = isl.isl_union_map_is_equal(self.ptr, arg1.ptr)
         return res
-    def is_strict_subset(self, arg1):
-        try:
-            if not arg1.__class__ is union_map:
-                arg1 = union_map(arg1)
-        except:
-            raise
-        res = isl.isl_union_map_is_strict_subset(self.ptr, arg1.ptr)
-        return res
-    def foreach_map(self, arg1):
-        exc_info = [None]
-        fn = CFUNCTYPE(c_int, c_void_p, c_void_p)
-        def cb_func(cb_arg0, cb_arg1):
-            cb_arg0 = map(ctx=self.ctx, ptr=cb_arg0)
-            try:
-                arg1(cb_arg0)
-            except:
-                import sys
-                exc_info[0] = sys.exc_info()
-                return -1
-            return 0
-        cb = fn(cb_func)
-        res = isl.isl_union_map_foreach_map(self.ptr, cb, None)
-        if exc_info[0] != None:
-            raise exc_info[0][0], exc_info[0][1], exc_info[0][2]
-        return res
 
-isl.isl_union_map_from_map.restype = c_void_p
-isl.isl_union_map_from_map.argtypes = [c_void_p]
 isl.isl_union_map_read_from_str.restype = c_void_p
 isl.isl_union_map_read_from_str.argtypes = [Context, c_char_p]
+isl.isl_union_map_from_basic_map.restype = c_void_p
+isl.isl_union_map_from_basic_map.argtypes = [c_void_p]
+isl.isl_union_map_from_map.restype = c_void_p
+isl.isl_union_map_from_map.argtypes = [c_void_p]
 isl.isl_union_map_affine_hull.restype = c_void_p
 isl.isl_union_map_polyhedral_hull.restype = c_void_p
 isl.isl_union_map_coalesce.restype = c_void_p
@@ -246,6 +268,8 @@ isl.isl_union_map_gist_domain.restype = c_void_p
 isl.isl_union_map_gist_range.restype = c_void_p
 isl.isl_union_map_intersect_domain.restype = c_void_p
 isl.isl_union_map_intersect_range.restype = c_void_p
+isl.isl_union_map_subtract_domain.restype = c_void_p
+isl.isl_union_map_subtract_range.restype = c_void_p
 isl.isl_union_map_apply_domain.restype = c_void_p
 isl.isl_union_map_apply_range.restype = c_void_p
 isl.isl_union_map_reverse.restype = c_void_p
@@ -407,6 +431,14 @@ class map(union_map):
             return union_map(self).is_equal(arg1)
         res = isl.isl_map_is_equal(self.ptr, arg1.ptr)
         return res
+    def is_disjoint(self, arg1):
+        try:
+            if not arg1.__class__ is map:
+                arg1 = map(arg1)
+        except:
+            return union_map(self).is_disjoint(arg1)
+        res = isl.isl_map_is_disjoint(self.ptr, arg1.ptr)
+        return res
     def is_single_valued(self):
         res = isl.isl_map_is_single_valued(self.ptr)
         return res
@@ -640,6 +672,10 @@ class union_set:
             self.ctx = keywords["ctx"]
             self.ptr = keywords["ptr"]
             return
+        if len(args) == 1 and args[0].__class__ is basic_set:
+            self.ctx = Context.getDefaultInstance()
+            self.ptr = isl.isl_union_set_from_basic_set(isl.isl_basic_set_copy(args[0].ptr))
+            return
         if len(args) == 1 and args[0].__class__ is set:
             self.ctx = Context.getDefaultInstance()
             self.ptr = isl.isl_union_set_from_set(isl.isl_set_copy(args[0].ptr))
@@ -659,9 +695,6 @@ class union_set:
         return res
     def __repr__(self):
         return 'isl.union_set("%s")' % str(self)
-    def identity(self):
-        res = isl.isl_union_set_identity(isl.isl_union_set_copy(self.ptr))
-        return union_map(ctx=self.ctx, ptr=res)
     def detect_equalities(self):
         res = isl.isl_union_set_detect_equalities(isl.isl_union_set_copy(self.ptr))
         return union_set(ctx=self.ctx, ptr=res)
@@ -772,12 +805,16 @@ class union_set:
         if exc_info[0] != None:
             raise exc_info[0][0], exc_info[0][1], exc_info[0][2]
         return res
+    def identity(self):
+        res = isl.isl_union_set_identity(isl.isl_union_set_copy(self.ptr))
+        return union_map(ctx=self.ctx, ptr=res)
 
+isl.isl_union_set_from_basic_set.restype = c_void_p
+isl.isl_union_set_from_basic_set.argtypes = [c_void_p]
 isl.isl_union_set_from_set.restype = c_void_p
 isl.isl_union_set_from_set.argtypes = [c_void_p]
 isl.isl_union_set_read_from_str.restype = c_void_p
 isl.isl_union_set_read_from_str.argtypes = [Context, c_char_p]
-isl.isl_union_set_identity.restype = c_void_p
 isl.isl_union_set_detect_equalities.restype = c_void_p
 isl.isl_union_set_affine_hull.restype = c_void_p
 isl.isl_union_set_polyhedral_hull.restype = c_void_p
@@ -790,6 +827,7 @@ isl.isl_union_set_intersect_params.restype = c_void_p
 isl.isl_union_set_gist.restype = c_void_p
 isl.isl_union_set_gist_params.restype = c_void_p
 isl.isl_union_set_apply.restype = c_void_p
+isl.isl_union_set_identity.restype = c_void_p
 isl.isl_union_set_free.argtypes = [c_void_p]
 isl.isl_union_set_to_str.argtypes = [c_void_p]
 isl.isl_union_set_to_str.restype = POINTER(c_char)
@@ -800,14 +838,14 @@ class set(union_set):
             self.ctx = keywords["ctx"]
             self.ptr = keywords["ptr"]
             return
-        if len(args) == 1 and args[0].__class__ is basic_set:
-            self.ctx = Context.getDefaultInstance()
-            self.ptr = isl.isl_set_from_basic_set(isl.isl_basic_set_copy(args[0].ptr))
-            return
         if len(args) == 1 and type(args[0]) == str:
             self.ctx = Context.getDefaultInstance()
             self.ptr = isl.isl_set_read_from_str(self.ctx, args[0])
             return
+        if len(args) == 1 and args[0].__class__ is basic_set:
+            self.ctx = Context.getDefaultInstance()
+            self.ptr = isl.isl_set_from_basic_set(isl.isl_basic_set_copy(args[0].ptr))
+            return
         raise Error
     def __del__(self):
         if hasattr(self, 'ptr'):
@@ -819,26 +857,6 @@ class set(union_set):
         return res
     def __repr__(self):
         return 'isl.set("%s")' % str(self)
-    def sample(self):
-        res = isl.isl_set_sample(isl.isl_set_copy(self.ptr))
-        return basic_set(ctx=self.ctx, ptr=res)
-    def detect_equalities(self):
-        res = isl.isl_set_detect_equalities(isl.isl_set_copy(self.ptr))
-        return set(ctx=self.ctx, ptr=res)
-    def affine_hull(self):
-        res = isl.isl_set_affine_hull(isl.isl_set_copy(self.ptr))
-        return basic_set(ctx=self.ctx, ptr=res)
-    def polyhedral_hull(self):
-        res = isl.isl_set_polyhedral_hull(isl.isl_set_copy(self.ptr))
-        return basic_set(ctx=self.ctx, ptr=res)
-    def union(self, arg1):
-        try:
-            if not arg1.__class__ is set:
-                arg1 = set(arg1)
-        except:
-            return union_set(self).union(arg1)
-        res = isl.isl_set_union(isl.isl_set_copy(self.ptr), isl.isl_set_copy(arg1.ptr))
-        return set(ctx=self.ctx, ptr=res)
     def intersect(self, arg1):
         try:
             if not arg1.__class__ is set:
@@ -874,6 +892,32 @@ class set(union_set):
             return union_set(self).apply(arg1)
         res = isl.isl_set_apply(isl.isl_set_copy(self.ptr), isl.isl_map_copy(arg1.ptr))
         return set(ctx=self.ctx, ptr=res)
+    def lexmin(self):
+        res = isl.isl_set_lexmin(isl.isl_set_copy(self.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def lexmax(self):
+        res = isl.isl_set_lexmax(isl.isl_set_copy(self.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def sample(self):
+        res = isl.isl_set_sample(isl.isl_set_copy(self.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
+    def detect_equalities(self):
+        res = isl.isl_set_detect_equalities(isl.isl_set_copy(self.ptr))
+        return set(ctx=self.ctx, ptr=res)
+    def affine_hull(self):
+        res = isl.isl_set_affine_hull(isl.isl_set_copy(self.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
+    def polyhedral_hull(self):
+        res = isl.isl_set_polyhedral_hull(isl.isl_set_copy(self.ptr))
+        return basic_set(ctx=self.ctx, ptr=res)
+    def union(self, arg1):
+        try:
+            if not arg1.__class__ is set:
+                arg1 = set(arg1)
+        except:
+            return union_set(self).union(arg1)
+        res = isl.isl_set_union(isl.isl_set_copy(self.ptr), isl.isl_set_copy(arg1.ptr))
+        return set(ctx=self.ctx, ptr=res)
     def is_empty(self):
         res = isl.isl_set_is_empty(self.ptr)
         return res
@@ -901,12 +945,14 @@ class set(union_set):
             return union_set(self).is_equal(arg1)
         res = isl.isl_set_is_equal(self.ptr, arg1.ptr)
         return res
-    def lexmin(self):
-        res = isl.isl_set_lexmin(isl.isl_set_copy(self.ptr))
-        return set(ctx=self.ctx, ptr=res)
-    def lexmax(self):
-        res = isl.isl_set_lexmax(isl.isl_set_copy(self.ptr))
-        return set(ctx=self.ctx, ptr=res)
+    def is_disjoint(self, arg1):
+        try:
+            if not arg1.__class__ is set:
+                arg1 = set(arg1)
+        except:
+            return union_set(self).is_disjoint(arg1)
+        res = isl.isl_set_is_disjoint(self.ptr, arg1.ptr)
+        return res
     def gist(self, arg1):
         try:
             if not arg1.__class__ is set:
@@ -945,15 +991,10 @@ class set(union_set):
         res = isl.isl_set_flatten(isl.isl_set_copy(self.ptr))
         return set(ctx=self.ctx, ptr=res)
 
-isl.isl_set_from_basic_set.restype = c_void_p
-isl.isl_set_from_basic_set.argtypes = [c_void_p]
 isl.isl_set_read_from_str.restype = c_void_p
 isl.isl_set_read_from_str.argtypes = [Context, c_char_p]
-isl.isl_set_sample.restype = c_void_p
-isl.isl_set_detect_equalities.restype = c_void_p
-isl.isl_set_affine_hull.restype = c_void_p
-isl.isl_set_polyhedral_hull.restype = c_void_p
-isl.isl_set_union.restype = c_void_p
+isl.isl_set_from_basic_set.restype = c_void_p
+isl.isl_set_from_basic_set.argtypes = [c_void_p]
 isl.isl_set_intersect.restype = c_void_p
 isl.isl_set_intersect_params.restype = c_void_p
 isl.isl_set_subtract.restype = c_void_p
@@ -961,6 +1002,11 @@ isl.isl_set_complement.restype = c_void_p
 isl.isl_set_apply.restype = c_void_p
 isl.isl_set_lexmin.restype = c_void_p
 isl.isl_set_lexmax.restype = c_void_p
+isl.isl_set_sample.restype = c_void_p
+isl.isl_set_detect_equalities.restype = c_void_p
+isl.isl_set_affine_hull.restype = c_void_p
+isl.isl_set_polyhedral_hull.restype = c_void_p
+isl.isl_set_union.restype = c_void_p
 isl.isl_set_gist.restype = c_void_p
 isl.isl_set_coalesce.restype = c_void_p
 isl.isl_set_identity.restype = c_void_p
